/*
Copyright (C) 2012 Trenz Electronic

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
*/
/*
 * one_wire.h
 *
 *      Author: Oleksandr Kiyenko
 */
#include "xgpio.h"
//#include "tmr_delay.h"

#ifndef ONE_WIRE_H_
#define ONE_WIRE_H_

volatile unsigned int *one_wire_gpio;
volatile unsigned int *one_wire_timer;
unsigned int ticks_in_mks;

#define GPIO_DATA_REG	0
#define GPIO_TRI_REG	1
#define TIMER_TCSR_REG	0
#define TIMER_TCR_REG	2
//#define TIMER_TCR_REG	5

#define ONE_WIRE_TRSTL	480
#define ONE_WIRE_RESP	70
#define ONE_WIRE_RSTE	410
#define ONE_WIRE_TLOW	6
#define ONE_WIRE_TMWR	54
#define ONE_WIRE_TRDS	9
#define ONE_WIRE_TMRD	45
#define ONE_WIRE_TREC	10

#define ONE_WIRE_CMD_SKIP_ROM	0xCC
#define ONE_WIRE_CMD_READ_ROM	0x33
#define ONE_WIRE_CMD_MATCH_ROM	0x55
#define ONE_WIRE_CMD_SEARCH_ROM	0xF0

#define DS2502_CMD_READ_MEMORY	0xF0
#define DS2502_CMD_READ_STATUS	0xAA
#define DS2502_CMD_READ_DATA	0xC3
#define DS2502_CMD_WRITE_MEMORY	0x0F
#define DS2502_CMD_WRITE_STATUS	0x55

int one_wire_init(int gpio_addr, int timer_addr, int timer_freq);
void one_wire_stop();
int one_wire_reset();
int one_wire_write_bit(u8 btw);
int one_wire_write_byte(u8 btw);
u8 one_wire_read_bit();
u8 one_wire_read_byte();
int one_wire_search(unsigned char* addrp, u8 defined);
int one_wire_family(unsigned char* addrp);
int mks_sleep(unsigned int mks);
u8 one_wire_crc8(u8 ov, u8 av);

#endif /* ONE_WIRE_H_ */
